import React, { useEffect, useRef, useMemo } from 'react';
import { Terminal } from '@xterm/xterm';
import { FitAddon } from '@xterm/addon-fit';
import { WebLinksAddon } from '@xterm/addon-web-links';
import '@xterm/xterm/css/xterm.css';

type FileSystemStructure = {
  [key: string]: string[] | string; 
};

interface TerminalProps {
  puzzle1Content: string;
  puzzle2Content: string;
  puzzle3Content: string;
}

const TerminalComponent: React.FC<TerminalProps> = ({ 
  puzzle1Content,
  puzzle2Content,
  puzzle3Content
}) => {
  const terminalRef = useRef<HTMLDivElement>(null);
  const xtermRef = useRef<Terminal | null>(null);
  const fitAddonRef = useRef<FitAddon | null>(null);
  const commandHistory = useRef<string[]>([]);
  const historyIndex = useRef<number>(-1);
  const currentLine = useRef<string>('');

  const fileSystem: FileSystemStructure = useMemo(() => ({
    '/': ['home', 'documents', 'etc', 'var', 'usr', 'README.txt', 'secret.txt'],
    '/home': ['user', 'guest'],
    '/home/user': ['puzzle_progress.txt'],
    '/home/guest': [],
    '/documents': ['report.doc', 'notes.txt'],
    '/etc': ['config', 'sys_data'],
    '/etc/config': ['network.conf', 'color_palette.config'],
    '/etc/sys_data': ['core_access.bin'],
    '/var': ['log', 'tmp'],
    '/var/log': ['system.log', 'critical_errors.log'],
    '/usr': ['bin', 'share'],
    '/usr/bin': ['common_commands.txt'],
    '/usr/share': ['docs', 'misc'],
    '/usr/share/docs': ['filesystem_layout.txt'],
    '/usr/share/misc': ['old_data.bak'],

    // File Contents
    '/README.txt': "Welcome, challenger. Your mission, should you choose to accept it...\n" +
                   "Is to find three numbers to unlock a combination lock.\n" +
                   "These numbers are hidden within this simulated filesystem.\n" +
                   "Use commands like 'ls', 'cd', and 'cat' to explore.\n" +
                   "A good starting point might be to understand the lay of the land: 'cat /usr/share/docs/filesystem_layout.txt'\n" +
                   "Good luck!",
    '/secret.txt': "SECURITY NOTICE:\n\nSystem access restricted to authorized personnel only.\nAll activities on this terminal are logged and monitored.\n\nNote to admins: The master backup codes have been moved to a more secure location.\nPlease refer to the new security protocol documentation.\n\nCheck system configuration in /etc and examine logs in /var/log if issues arise.",
    '/usr/share/docs/filesystem_layout.txt': "System Architecture Overview:\n" +
                                            "- /home: User-specific files.\n" +
                                            "- /etc: System configuration files and critical data.\n" +
                                            "  |- /config: Display and network settings.\n" +
                                            "  |- /sys_data: Core system binary data.\n" +
                                            "- /var: Variable data, such as logs and temporary files.\n" +
                                            "  |- /log: System logs and error reports.\n" +
                                            "- /usr: System-wide shared resources and executables (read-only for you!).\n",

    // Puzzle 1: Number Base
    '/etc/sys_data/core_access.bin': puzzle1Content,

    // Puzzle 2: Hex Code (Color)
    '/etc/config/color_palette.config': puzzle2Content,

    // Puzzle 3: Log File Analysis
    '/var/log/critical_errors.log': puzzle3Content,

    // Home directory files with hints
    '/home/user/puzzle_progress.txt': "My notes on the combination lock:\n- First number: __ (Check core system data)\n- Second number: __ (UI color settings have relevant info)\n- Third number: __ (Error logs contain critical values)\n\nReminder: Always convert between number formats as needed!",
    
    // Document files with hints
    '/documents/report.doc': "Project Phoenix - Q3 Financial Report Summary\n\nStrictly Confidential\n\nNote: System core access requires binary authentication module as referenced in maintenance guide section 4.2.1",
    '/documents/notes.txt': "Grocery List:\n- Milk\n- Eggs\n- Bread\n- Mystery item #42 (NOT the combination number!)\n- Remember to check system logs for maintenance schedule\n\nReminders:\n- Review critical error logs for system failures\n- Update color palette settings per design team request",
    
    // Network config with hint
    '/etc/config/network.conf': "HOSTNAME=ixabatasha-virtual-env\nIP_ADDR=10.1.33.7\nGATEWAY=10.1.33.1\nDNS_SERVER=8.8.8.8\n\n# Note: Display color palette in the same directory needs to be accessible for UI integration",
    
    // System log with hint
    '/var/log/system.log': "[INFO] System startup sequence initiated.\n[INFO] All core services loaded successfully.\n[WARN] Low virtual memory. Consider rebooting non-essential modules.\n[INFO] User 'root' logged in from console.\n[NOTICE] Combination lock module initialized with secure parameters.\n[DEBUG] Numbers not stored in plaintext for security reasons.\n[INFO] Memory address values in binary format stored in /etc/sys_data\n[WARN] Check critical_errors.log for security daemon failures",
    
    '/usr/bin/common_commands.txt': "Available commands: ls, cd, cat, clear, help (for README)\n\nImportant system paths:\n- Main config: /etc/config/\n- System data: /etc/sys_data/\n- Error logs: /var/log/",
    
    '/usr/share/misc/old_data.bak': "ARCHIVE FILE: Contains outdated research notes from Project Chimera. Access restricted.\n\nPossible combination values tested:\n- 12-45-78 (Failed)\n- 24-68-99 (Failed)\n- 11-22-33 (Failed)\n\nNOTE: New secure generation algorithm implemented. Old combinations no longer valid.\n\nCurrent security relies on: core_access biometrics, color_palette UI safeguards, and error logging thresholds."
  }), [puzzle1Content, puzzle2Content, puzzle3Content]);
  const currentPath = useRef<string>('/');

  const prompt = () => {
    if (xtermRef.current) {
      xtermRef.current.write(`\r\nroot@ixabatasha.life:${currentPath.current}# `);
    }
  };

  const processCommand = (command: string) => {
    if (!xtermRef.current) return;
    const [cmd, ...args] = command.trim().split(' ');

    switch (cmd) {
      case 'ls':
        const currentDirContent = fileSystem[currentPath.current];
        if (Array.isArray(currentDirContent)) {
          const displayItems = currentDirContent.map(item => {
            let itemFullPath = (currentPath.current === '/' ? '' : currentPath.current) + '/' + item;
            if (itemFullPath !== '/' && itemFullPath.endsWith('/')) itemFullPath = itemFullPath.slice(0, -1);

            if (Array.isArray(fileSystem[itemFullPath])) {
              return item + '/';
            }
            return item;
          });
          xtermRef.current.write('\r\n' + displayItems.join('\t'));
        } else {
          xtermRef.current.write(`\r\nls: cannot access '${currentPath.current}': Not a directory`);
        }
        break;
      case 'cd':
        const targetDir = args[0];
        if (!targetDir) {
          // cd to root
          currentPath.current = '/';
        } else if (targetDir === '..') {
          const parts = currentPath.current.split('/').filter(p => p);
          parts.pop();
          currentPath.current = '/' + parts.join('/');
          if (currentPath.current === '//') currentPath.current = '/';
        } else {
          let newPath: string;
          if (targetDir.startsWith('/')) {
            newPath = targetDir;
          } else {
            newPath = (currentPath.current === '/' ? '' : currentPath.current) + '/' + targetDir;
          }
          // Normalize path (remove trailing slashes unless it's the root)
          if (newPath !== '/' && newPath.endsWith('/')) {
            newPath = newPath.slice(0, -1);
          }

          if (fileSystem[newPath] !== undefined) {
            if (Array.isArray(fileSystem[newPath])) {
              currentPath.current = newPath;
            } else {
              xtermRef.current.write(`\r\ncd: ${targetDir}: Not a directory`);
            }
          } else {
            xtermRef.current.write(`\r\ncd: ${targetDir}: No such file or directory`);
          }
        }
        break;
      case 'cat':
        const targetFileArg = args[0];
        if (!targetFileArg) {
          xtermRef.current.write('\r\ncat: missing file operand');
          break;
        }
        
        let fullTargetPath: string;
        if (targetFileArg.startsWith('/')) {
          fullTargetPath = targetFileArg;
        } else {
          fullTargetPath = (currentPath.current === '/' ? '' : currentPath.current) + '/' + targetFileArg;
        }
        // Normalize path
        if (fullTargetPath !== '/' && fullTargetPath.endsWith('/')) {
          fullTargetPath = fullTargetPath.slice(0, -1);
        }

        const fileContent = fileSystem[fullTargetPath];

        if (fileContent === undefined) {
          xtermRef.current.write(`\r\ncat: ${targetFileArg}: No such file or directory`);
        } else if (Array.isArray(fileContent)) {
          xtermRef.current.write(`\r\ncat: ${targetFileArg}: Is a directory`);
        } else if (typeof fileContent === 'string') {
          xtermRef.current.write('\r\n' + fileContent.split('\n').join('\r\n'));
        } else {
          // Should not happen with current structure
          xtermRef.current.write(`\r\ncat: ${targetFileArg}: Cannot read this type of file`);
        }
        break;
      case 'clear':
        xtermRef.current.clear();
        break;
      case 'help':
        // Display available commands and tips
        const helpText = 
          "Available Commands:\r\n" +
          "  ls                    List directory contents\r\n" +
          "  cd <directory>        Change directory\r\n" +
          "  cat <file>            Display file contents\r\n" +
          "  clear                 Clear the screen\r\n" +
          "  help                  Display this help message\r\n\r\n" +
          "Tips:\r\n" +
          "  - Use Tab for command/path completion\r\n" +
          "  - Up/Down arrows navigate command history\r\n" +
          "  - Type 'cat /usr/bin/common_commands.txt' for more system information\r\n" +
          "  - Read the README.txt for mission details";
        
        xtermRef.current.write('\r\n' + helpText);
        break;
      case '': // Handle empty command
        break;
      default:
        xtermRef.current.write(`\r\ncommand not found: ${cmd}\r\nTry 'help' for a list of available commands`);
        break;
    }
    prompt();
  };

  useEffect(() => {
    if (terminalRef.current && !xtermRef.current) {
      const term = new Terminal({
        cursorBlink: true,
        fontFamily: 'monospace',
        fontSize: 14,
        theme: {
          background: '#1e1e1e',
          foreground: '#d4d4d4',
          cursor: '#d4d4d4',
        },
        allowTransparency: true,
        scrollback: 1000,
        rightClickSelectsWord: true
      });
      const fit = new FitAddon();
      const webLinks = new WebLinksAddon();

      xtermRef.current = term;
      fitAddonRef.current = fit;

      term.loadAddon(fit);
      term.loadAddon(webLinks);
      term.open(terminalRef.current);
      fit.fit();

      // Enhanced paste handler for Ctrl+V and Cmd+V
      term.attachCustomKeyEventHandler((e) => {
        // Handle paste with Ctrl+V (Windows/Linux) or Cmd+V (Mac)
        if ((e.ctrlKey || e.metaKey) && e.key === 'v') {
          e.preventDefault();
          navigator.clipboard.readText().then(text => {
            // Process the pasted text character by character to update currentLine
            const cleanText = text.replace(/\r\n/g, '\n').replace(/\r/g, '\n');
            for (const char of cleanText) {
              if (char === '\n') {
                // Handle newline as Enter key
                if (currentLine.current.trim()) {
                  commandHistory.current.push(currentLine.current);
                }
                historyIndex.current = commandHistory.current.length;
                processCommand(currentLine.current);
                currentLine.current = '';
              } else {
                // Add printable characters to current line and display
                term.write(char);
                currentLine.current += char;
              }
            }
          }).catch(err => {
            console.warn('Failed to read clipboard:', err);
          });
          return false;
        }
        
        // Handle copy with Ctrl+C or Cmd+C (when text is selected)
        if ((e.ctrlKey || e.metaKey) && e.key === 'c') {
          const selection = term.getSelection();
          if (selection) {
            navigator.clipboard.writeText(selection).catch(err => {
              console.warn('Failed to write to clipboard:', err);
            });
          }
          return true; // Don't prevent default for copy
        }
        
        return true;
      });

      // Add right-click context menu support
      term.element?.addEventListener('contextmenu', (e) => {
        e.preventDefault();
        
        // Create context menu
        const existingMenu = document.getElementById('terminal-context-menu');
        if (existingMenu) {
          existingMenu.remove();
        }
        
        const menu = document.createElement('div');
        menu.id = 'terminal-context-menu';
        menu.style.cssText = `
          position: fixed;
          top: ${e.clientY}px;
          left: ${e.clientX}px;
          background: #2d2d2d;
          border: 1px solid #555;
          border-radius: 4px;
          padding: 4px 0;
          box-shadow: 0 2px 8px rgba(0,0,0,0.3);
          z-index: 1000;
          font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, sans-serif;
          font-size: 14px;
          min-width: 120px;
        `;
        
        const selection = term.getSelection();
        
        // Copy option (only if text is selected)
        if (selection) {
          const copyItem = document.createElement('div');
          copyItem.textContent = 'Copy';
          copyItem.style.cssText = `
            padding: 8px 16px;
            color: #fff;
            cursor: pointer;
            transition: background-color 0.1s;
          `;
          copyItem.addEventListener('mouseenter', () => {
            copyItem.style.backgroundColor = '#404040';
          });
          copyItem.addEventListener('mouseleave', () => {
            copyItem.style.backgroundColor = 'transparent';
          });
          copyItem.addEventListener('click', () => {
            navigator.clipboard.writeText(selection).catch(err => {
              console.warn('Failed to write to clipboard:', err);
            });
            menu.remove();
          });
          menu.appendChild(copyItem);
        }
        
        // Paste option
        const pasteItem = document.createElement('div');
        pasteItem.textContent = 'Paste';
        pasteItem.style.cssText = `
          padding: 8px 16px;
          color: #fff;
          cursor: pointer;
          transition: background-color 0.1s;
        `;
        pasteItem.addEventListener('mouseenter', () => {
          pasteItem.style.backgroundColor = '#404040';
        });
        pasteItem.addEventListener('mouseleave', () => {
          pasteItem.style.backgroundColor = 'transparent';
        });
        pasteItem.addEventListener('click', () => {
          navigator.clipboard.readText().then(text => {
            // Process the pasted text character by character to update currentLine
            const cleanText = text.replace(/\r\n/g, '\n').replace(/\r/g, '\n');
            for (const char of cleanText) {
              if (char === '\n') {
                // Handle newline as Enter key
                if (currentLine.current.trim()) {
                  commandHistory.current.push(currentLine.current);
                }
                historyIndex.current = commandHistory.current.length;
                processCommand(currentLine.current);
                currentLine.current = '';
              } else {
                // Add printable characters to current line and display
                term.write(char);
                currentLine.current += char;
              }
            }
          }).catch(err => {
            console.warn('Failed to read clipboard:', err);
          });
          menu.remove();
        });
        menu.appendChild(pasteItem);
        
        document.body.appendChild(menu);
        
        // Remove menu when clicking elsewhere
        const removeMenu = (event: MouseEvent) => {
          if (!menu.contains(event.target as Node)) {
            menu.remove();
            document.removeEventListener('click', removeMenu);
          }
        };
        
        // Use setTimeout to avoid immediate removal
        setTimeout(() => {
          document.addEventListener('click', removeMenu);
        }, 0);
      });

      term.write('Welcome to Ixabata-shell!\r\n');
      term.write('Type a command to begin. (Try ls, cd, cat)\r\n');
      prompt();

      term.onKey(({ key, domEvent }) => {
        if (!xtermRef.current) return;

        const printable = !domEvent.altKey && !domEvent.ctrlKey && !domEvent.metaKey;

        if (domEvent.key === 'Enter') {
          if (currentLine.current.trim()) {
            commandHistory.current.push(currentLine.current);
          }
          historyIndex.current = commandHistory.current.length;
          processCommand(currentLine.current);
          currentLine.current = '';
        } else if (domEvent.key === 'Backspace') {
          if (currentLine.current.length > 0) {
            xtermRef.current.write('\b \b'); // Erase character
            currentLine.current = currentLine.current.slice(0, -1);
          }
        } else if (domEvent.key === 'ArrowUp') {
          if (commandHistory.current.length > 0 && historyIndex.current > 0) {
            historyIndex.current--;
            // Clear current line and write prompt + history item
            xtermRef.current.write('\x1b[2K\r');
            xtermRef.current.write(`root@ixabatasha.life.life:${currentPath.current}# `);
            xtermRef.current.write(commandHistory.current[historyIndex.current]);
            currentLine.current = commandHistory.current[historyIndex.current];
          }
        } else if (domEvent.key === 'ArrowDown') {
          if (commandHistory.current.length > 0 && historyIndex.current < commandHistory.current.length - 1) {
            historyIndex.current++;
            // Clear current line and write prompt + history item
            xtermRef.current.write('\x1b[2K\r');
            xtermRef.current.write(`root@ixabatasha.life.life:${currentPath.current}# `);
            xtermRef.current.write(commandHistory.current[historyIndex.current]);
            currentLine.current = commandHistory.current[historyIndex.current];
          } else if (historyIndex.current === commandHistory.current.length - 1) {
            historyIndex.current++;
            // Clear current line and write just the prompt
            xtermRef.current.write('\x1b[2K\r');
            xtermRef.current.write(`root@ixabatasha.life.life:${currentPath.current}# `);
            currentLine.current = "";
          }
        } else if (domEvent.key === 'Tab') {
          domEvent.preventDefault(); // Prevent default tab behavior (focus change)
          const inputParts = currentLine.current.split(' ');
          const currentArgument = inputParts.pop() || '';
          const commandPart = inputParts.join(' ');

          let suggestions: string[] = [];
          const availableCommands = ['ls', 'cd', 'cat', 'clear', 'help']; 

          // If we're completing a command (first word)
          if (inputParts.length === 0 && !currentLine.current.includes(' ')) {
            suggestions = availableCommands.filter(c => c.startsWith(currentArgument));
          } else {
            // For path completion (after a command)
            const command = inputParts[0];
            
            // Only complete paths for commands that accept file paths
            if (['ls', 'cd', 'cat'].includes(command)) {
              // Handle absolute paths
              if (currentArgument.startsWith('/')) {
                const pathParts = currentArgument.split('/');
                const lastPart = pathParts.pop() || '';
                const basePath = pathParts.join('/') || '/';
                
                if (fileSystem[basePath] && Array.isArray(fileSystem[basePath])) {
                  suggestions = (fileSystem[basePath] as string[])
                    .filter(item => item.startsWith(lastPart))
                    .map(item => {
                      // Return the full path for absolute completions
                      return basePath === '/' ? '/' + item : basePath + '/' + item;
                    });
                }
              } else {
                // Handle relative paths
                const currentDirItems = Array.isArray(fileSystem[currentPath.current]) 
                  ? fileSystem[currentPath.current] as string[] 
                  : [];
                suggestions = currentDirItems.filter(item => item.startsWith(currentArgument));
              }
            }
          }

          if (suggestions.length === 1) {
            const suggestion = suggestions[0];
            let resolvedSuggestionPath = suggestion;
            
            // Handle path resolution for the suggestion
            if (currentArgument.startsWith('/')) {
              // For absolute paths, the suggestion is already the full path
              resolvedSuggestionPath = suggestion;
            } else {
              // For relative paths, use the current directory
              resolvedSuggestionPath = (currentPath.current === '/' ? '' : currentPath.current) + '/' + suggestion;
            }
            
            // Normalize path
            if (resolvedSuggestionPath !== '/' && resolvedSuggestionPath.endsWith('/')) {
              resolvedSuggestionPath = resolvedSuggestionPath.slice(0, -1);
            }
            
            const isDirectory = Array.isArray(fileSystem[resolvedSuggestionPath]);
            
            let completion: string;
            if (currentArgument.startsWith('/')) {
              // For absolute paths, replace the entire argument
              completion = suggestion + (isDirectory ? '/' : ' ');
              xtermRef.current.write('\x1b[2K\r'); // Clear the line
              xtermRef.current.write(`root@ixabatasha.life:${currentPath.current}# `);
              xtermRef.current.write((commandPart ? commandPart + ' ' : '') + completion);
              currentLine.current = (commandPart ? commandPart + ' ' : '') + completion;
            } else {
              // For relative paths, append to the existing argument
              completion = suggestion + (isDirectory ? '/' : ' ');
              xtermRef.current.write(completion.substring(currentArgument.length));
              currentLine.current = (commandPart ? commandPart + ' ' : '') + completion;
            }
            
            if (!isDirectory || !completion.endsWith('/')) {
              currentLine.current = currentLine.current.trimEnd();
            }
          } else if (suggestions.length > 1) {
            // Show all possible completions in a bash-like format
            xtermRef.current.write('\r\n');
            
            // For display purposes, show just the item names, not full paths
            const displaySuggestions = suggestions.map(s => {
              if (currentArgument.startsWith('/')) {
                return s.split('/').pop() || s;
              }
              return s;
            });
            
            // Calculate the longest common prefix
            let commonPrefix = '';
            if (displaySuggestions.length > 0) {
              const first = displaySuggestions[0];
              for (let i = 0; i < first.length; i++) {
                if (displaySuggestions.every(s => s[i] === first[i])) {
                  commonPrefix += first[i];
                } else {
                  break;
                }
              }
            }

            // If there's a common prefix, complete up to that point
            if (commonPrefix.length > (currentArgument.startsWith('/') ? currentArgument.split('/').pop()?.length || 0 : currentArgument.length)) {
              if (currentArgument.startsWith('/')) {
                const pathParts = currentArgument.split('/');
                pathParts.pop();
                const newPath = pathParts.join('/') + '/' + commonPrefix;
                xtermRef.current.write('\x1b[2K\r'); // Clear the line
                xtermRef.current.write(`root@ixabatasha.life:${currentPath.current}# `);
                xtermRef.current.write((commandPart ? commandPart + ' ' : '') + newPath);
                currentLine.current = (commandPart ? commandPart + ' ' : '') + newPath;
              } else {
                xtermRef.current.write(commonPrefix.substring(currentArgument.length));
                currentLine.current = (commandPart ? commandPart + ' ' : '') + commonPrefix;
              }
            } else {
              // Display all options in columns
              const maxLength = Math.max(...displaySuggestions.map(s => s.length)) + 2;
              const columns = Math.floor(term.cols / maxLength);
              const rows = Math.ceil(displaySuggestions.length / columns);
              
              for (let row = 0; row < rows; row++) {
                let line = '';
                for (let col = 0; col < columns; col++) {
                  const index = row + (col * rows);
                  if (index < displaySuggestions.length) {
                    const item = displaySuggestions[index];
                    const fullPath = suggestions[index];
                    const isDir = Array.isArray(fileSystem[fullPath.startsWith('/') ? fullPath : (currentPath.current === '/' ? '' : currentPath.current) + '/' + fullPath]);
                    line += item + (isDir ? '/' : '') + ' '.repeat(maxLength - item.length - (isDir ? 1 : 0));
                  }
                }
                xtermRef.current.write(line + '\r\n');
              }
              
              // Reprint the prompt and current input
              prompt();
              xtermRef.current.write(currentLine.current);
            }
          }
          // If no suggestions, do nothing
        } else if (printable && key.length === 1) { // Ensure it's a single character, not a special key like 'Shift'
          xtermRef.current.write(key);
          currentLine.current += key;
        }
      });

      // Handle resize
      const handleResize = () => {
        fitAddonRef.current?.fit();
      };
      window.addEventListener('resize', handleResize);

      return () => {
        window.removeEventListener('resize', handleResize);
        term.dispose();
        xtermRef.current = null;
      };
    }
  }, []);

  return <div ref={terminalRef} style={{ width: '100%', height: '100%' }} />;
};

export default TerminalComponent; 